def deep_merge(src_dict, merge_dict):
    for k, v in merge_dict.iteritems():
        if (k in src_dict and isinstance(src_dict[k], dict)):
            deep_merge(src_dict[k], merge_dict[k])
        else:
            src_dict[k] = merge_dict[k]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Extracted from django.utils.functionnal.
BSD license.
"""

import operator


empty = object()


def new_method_proxy(func):
    def inner(self, *args):
        if self._wrapped is empty:
            self._setup()
        return func(self._wrapped, *args)
    return inner


class LazyObject(object):
    """
    A wrapper for another class that can be used to delay instantiation of the
    wrapped class.

    By subclassing, you have the opportunity to intercept and alter the
    instantiation. If you don't need to do that, use SimpleLazyObject.
    """

    # Avoid infinite recursion when tracing __init__ (#19456).
    _wrapped = None

    def __init__(self):
        self._wrapped = empty

    __getattr__ = new_method_proxy(getattr)

    def __setattr__(self, name, value):
        if name == "_wrapped":
            # Assign to __dict__ to avoid infinite __setattr__ loops.
            self.__dict__["_wrapped"] = value
        else:
            if self._wrapped is empty:
                self._setup()
            setattr(self._wrapped, name, value)

    def __delattr__(self, name):
        if name == "_wrapped":
            raise TypeError("can't delete _wrapped.")
        if self._wrapped is empty:
            self._setup()
        delattr(self._wrapped, name)

    def _setup(self):
        """
        Must be implemented by subclasses to initialise the wrapped object.
        """
        raise NotImplementedError

    # Introspection support
    __dir__ = new_method_proxy(dir)

    # Dictionary methods support
    __getitem__ = new_method_proxy(operator.getitem)
    __setitem__ = new_method_proxy(operator.setitem)
    __delitem__ = new_method_proxy(operator.delitem)

    __len__ = new_method_proxy(len)
    __contains__ = new_method_proxy(operator.contains)



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from iziconf.utils.functional import LazyObject, empty


class LazySettings(LazyObject):
    """A lazy proxy for either global Simpleconf settings or a custom settings object.

    The user can manually configure settings prior to using them. Otherwise,
    iziconf uses the settings module pointed to by ``conffile``.
    """
    def __init__(self, settings_cls, settings_module, *args, **kwargs):
        self.__dict__['_settings_cls'] = settings_cls
        self.__dict__['_settings_module'] = settings_module
        self.__dict__['_args'] = args
        self.__dict__['_kwargs'] = kwargs

        super(LazySettings, self).__init__()

    def _setup(self, name=None):
        """
        Load the settings pointed to by conffile. This is used the first time
        we need any settings at all, if the user has not previously configured
        the settings manually.
        """
        self._wrapped = self._settings_cls(self._settings_module, *self._args, **self._kwargs)

    def __getattr__(self, name):
        if self._wrapped is empty:
            self._setup(name)
        return getattr(self._wrapped, name)

    def configure(self, settings_module):
        """ Called to manually configure the settings.
        The 'default_settings' parameter sets where to retrieve any unspecified values from (its
        argument must support attribute access (__getattr__)).

        :param settings_module: The path to the iziconf.ini configuration file.
 Only options declared in :mod:`iziconf.options.global_settings` will be parsed.


        """
        if self._wrapped is not empty:
            raise RuntimeError('Settings already configured.')
        self._wrapped = self._settings_cls(settings_module, *self._args, **self._kwargs)

    def reconfigure(self, settings_module, **kw_options):
        """
        Called to manually change the options file. It will close all previous
        session, dispose engine and recreate them with the new configuration
        file.
        """
        # Never been configured
        if self._wrapped is not empty:
            if 'dispose_function' in kw_options:
                dispose = kw_options.pop('dispose_function')
                dispose()
            self._wrapped = empty
        self.configure(settings_module, **kw_options)

    @property
    def configured(self):
        """
        Returns ``True`` if the settings have already been configured.
        """
        return self._wrapped is not empty



from __future__ import absolute_import, division, print_function

import yaml
from iziconf.utils.dict import deep_merge


class YamlSettings(object):
    def __init__(self, config_path, defaults, callback=None):
        """
        Configure iziconf.

        Get global settings (defaults) and override them with ini-file-defined
        custom settings.
        """
        self._config = {}
        try:
            with open(config_path, "rb") as f:
                self._config = yaml.load(f.read())
        except IOError:
            pass
        print(defaults)
        deep_merge(defaults, self._config)
        self._config = defaults
        print(self._config)
        # Read config
        self.configure('logger', 'log', self._config)

        if callback:
            callback(self)

    def configure(self, section, attr=None, conf=None):
        """Configure all settings from a given section."""
        if attr is None:
            attr = section

        if not conf:
            conf = self._config
        settings = conf[section]

        setattr(self, attr, settings)



# -*- coding: utf-8 -*-
from __future__ import absolute_import
import os
from os.path import isfile

from iziconf.lazy_settings import LazySettings
from iziconf.base_settings import YamlSettings
from iziconf.global_settings import global_settings
from iziconf.utils.singleton import Singleton


__author__ = 'Antoine Legrand'
__email__ = '2t.antoine@gmail.com'
__version__ = '0.0.2'


class Simpleconf(object):
    __metaclass__ = Singleton
    _default_conf = {"envvar_name": "SIMPLECONF_SETTINGS_FILE",
                     "settings_file": "/etc/SIMPLECONF/iziconf.yaml",
                     "callback": None}

    def _settings_file(self, conf=None):
        if not conf:
            conf = self._default_conf
        try:
            conffile = os.environ[conf['envvar_name']]
            if not isfile(conffile):
                raise KeyError
            return conffile
        except KeyError:
            # USE DEFAULT PATH
            return conf['settings_file']

    def __init__(self, conf=_default_conf):
        conffile = self._settings_file(conf)
        self.settings = LazySettings(YamlSettings, conffile,
                                     global_settings, callback=conf['callback'])
